import numpy, matplotlib.pyplot as plt
from numba import cuda
from utils import expSpace

def gen_angels_to_pick(n, plot = False):
    if n % 2 == 0:
        raise "n%2==0"
    N = int((n + 3) / 2)
    X1 = expSpace(0.0, numpy.pi/2.0, N)
    X3 = expSpace(numpy.pi/2.0, numpy.pi, N)
    X3 = -1*X3 + 3.0*numpy.pi/2.0

    anglestopick = numpy.concatenate((X1, X3), axis=None)
    anglestopick = numpy.unique(anglestopick) # Vigyázni vele!
    anglestopick = anglestopick[1:-1]

    if plot:
        Y = numpy.zeros(n)
        plt.plot(anglestopick, Y, '|')
        plt.show()
    
    return anglestopick

def angles_alap(anglestopick, plot = False):
    alpha_arr = numpy.array([])
    beta_arr = numpy.array([])

    for alpha in anglestopick:
        for beta in anglestopick:
            if alpha + beta < numpy.pi and beta >= alpha and alpha != 0.0: # vigyázni
                alpha_arr = numpy.insert(alpha_arr, 0, alpha)
                beta_arr = numpy.insert(beta_arr, 0, beta)

    tompa_beta_arr = beta_arr[beta_arr > numpy.pi]
    tompa_beta_mpi_arr = tompa_beta_arr - numpy.pi/2
    hegyes_beta_arr = beta_arr[beta_arr <= numpy.pi]
    tompa_alpha_arr = alpha_arr[beta_arr > numpy.pi]
    hegyes_alpha_arr = alpha_arr[beta_arr <= numpy.pi]

    tghB = numpy.tan(hegyes_beta_arr)
    tghA = numpy.tan(hegyes_alpha_arr)
    tgtB_mpi = numpy.tan(tompa_beta_mpi_arr)
    tgtA = numpy.tan(tompa_alpha_arr)

    # hegyes
    sztgh = tghB*tghA
    otgh = tghA + tghB
    hCy = sztgh/otgh
    hCx = hCy / tghA

    # tompa
    sztgt = tgtB_mpi*tgtA
    mtgt = 1.0 - sztgt
    tCy = tgtA / mtgt
    tCx = tCy / tgtA

    Cy = numpy.concatenate((hCy, tCy), axis=None)
    Cx = numpy.concatenate((hCx, tCx), axis=None)

    return Cx, Cy


def angles_ratet(anglestopick, plot = False):
    alpha_arr = numpy.array([])
    beta_arr = numpy.array([])

    for alpha in anglestopick:
        for beta in anglestopick:
            if alpha + beta < numpy.pi and alpha != 0.0: # vigyázni
                alpha_arr = numpy.insert(alpha_arr, 0, alpha)
                beta_arr = numpy.insert(beta_arr, 0, beta)

    tompa_beta_arr = beta_arr[beta_arr > numpy.pi]
    tompa_beta_mpi_arr = tompa_beta_arr - numpy.pi/2
    hegyes_beta_arr = beta_arr[beta_arr <= numpy.pi]
    tompa_alpha_arr = alpha_arr[beta_arr > numpy.pi]
    hegyes_alpha_arr = alpha_arr[beta_arr <= numpy.pi]

    tghB = numpy.tan(hegyes_beta_arr)
    tghA = numpy.tan(hegyes_alpha_arr)
    tgtB_mpi = numpy.tan(tompa_beta_mpi_arr)
    tgtA = numpy.tan(tompa_alpha_arr)

    # hegyes
    sztgh = tghB*tghA
    otgh = tghA + tghB
    hCy = sztgh/otgh
    hCx = hCy / tghA

    # tompa
    sztgt = tgtB_mpi*tgtA
    mtgt = 1.0 - sztgt
    tCy = tgtA / mtgt
    tCx = tCy / tgtA

    Ey = numpy.concatenate((hCy, tCy), axis=None)
    Ex = numpy.concatenate((hCx, tCx), axis=None)

    cos = numpy.cos(anglestopick)
    sin = numpy.sin(anglestopick)

    Dx = numpy.outer(numpy.full(anglestopick.size, 1.0), Ex).flatten()
    Dy = numpy.outer(cos, Ey).flatten()
    Dz = numpy.outer(sin, Ey).flatten()

    return Dx, Dy, Dz